package cn.com.qmth.examcloud.web.interceptor;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.servlet.HandlerInterceptor;

import com.codahale.metrics.ConsoleReporter;
import com.codahale.metrics.Meter;
import com.codahale.metrics.MetricRegistry;

import cn.com.qmth.examcloud.web.enums.HttpServletRequestAttribute;
import cn.com.qmth.examcloud.web.support.ApiInfo;

/**
 * 统计拦截器
 *
 * @author WANGWEI
 * @date 2019年7月24日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class StatisticInterceptor implements HandlerInterceptor {

	private Map<ApiInfo, Meter> meterMap = new HashMap<>();

	private MetricRegistry metrics = new MetricRegistry();

	ConsoleReporter reporter = ConsoleReporter.forRegistry(metrics).convertRatesTo(TimeUnit.SECONDS)
			.convertDurationsTo(TimeUnit.MILLISECONDS).build();

	public StatisticInterceptor() {
		reporter.start(1, TimeUnit.SECONDS);// 1s报道一次
	}

	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response,
			Object handler) throws Exception {
		ApiInfo apiInfo = (ApiInfo) request
				.getAttribute(HttpServletRequestAttribute.$_API_INFO.name());
		if (null == apiInfo) {
			return true;
		}

		Meter meter = meterMap.get(apiInfo);
		if (null == meter) {
			synchronized (this) {
				meter = meterMap.get(apiInfo);
				if (null == meter) {
					meter = metrics.meter(apiInfo.getMapping());
					meterMap.put(apiInfo, meter);
				}
			}
		}

		meter.mark();
		return true;
	}

	@Override
	public void afterCompletion(HttpServletRequest request, HttpServletResponse response,
			Object handler, Exception ex) throws Exception {

	}

}
